const apiai = require("apiai");
const Util = require('./util');

var Raven = require('raven');
Raven.config('https://7a1942e8017b45cd9ad993dcadecfe23@sentry.io/1263148').install();


const NAO_ENTENDI = 'naoentendi';
const NAO_ENTENDIVEL = 'nao_entendivel'

module.exports = function (config) {


    let resolver = {};

    const AsksuiteTranslator = require('./translator')(config);
    const AsksuiteUtil = require('./asksuite.util')(config);
    const KeywordMatcher = require('./keyword_interpreter/keyword-matcher')(config)


    const DialogFlow = apiai(config.apiAI_clientAcessToken);

    resolver.resolve = async function resolveText(userId, text, companyIntents, language, companyId) {

        return new Promise(async (resolve, reject) => {

            let intents = companyIntents;

            if (text.length < 80) {
                let lang =  AsksuiteUtil.defineLanguage(language);

                if (lang != "pt-br"){
                    text = await AsksuiteTranslator.googleTranslateIfNeed(text, lang)
                }

                let msg = Util.replaceIfIsPhrase(text.toLowerCase());

                console.log(msg);

                //se for feito o tratamento da mensagem e retornar null ou vazio, manda o texto que o cliente mandou mesmo.
                if (!msg || (msg != null && msg == '')){
                    msg = text.toLowerCase();
                }


                var request = DialogFlow.textRequest(msg, {
                    sessionId: userId,
                    resetContexts: true
                });

                request.on('response', function (response) {

                    if (intents != null){

                        let intentBot = intents.find(item => {
                            return item.intent == response.result.metadata.intentName;
                        });


                        //caso venha um não entendi do dialogflow, vou verificar se não é o caso de "ahhhhhhhhhhhh" ou hhh ???
                        if (response.result.metadata.intentName == NAO_ENTENDI){
                            //quer dizer é uma ou duas letras aleatórias
                            let isNonEntendible = (msg.length <= 2 && !Util.isNumeric(msg)) || Util.existsletterCount(msg, 3)
                            //não entendeu
                            if (isNonEntendible){
                                console.log("caiu aqui no não entendivel");
                                intentBot = intents.find(item => {
                                    return item.intent == NAO_ENTENDIVEL ;
                                });
                            }
                        }

                        if (intentBot) {
                            let cloneIntent = JSON.parse(JSON.stringify(intentBot));
                            Object.assign(cloneIntent, response.result.parameters);
                            resolve( cloneIntent)

                        } else {

                            Raven.captureMessage('Empresa não tem intent', {user : {userId: userId, text: text, companyIntents : companyIntents, language: language, intentResult: response.result.metadata.intentName }}, function (err, eventId) {
                                // The message has now been sent to Sentry
                            });

                            let intentLargeFixedName = NAO_ENTENDI;
                            let intentBot = intents.find(item => {
                                return item.intent == intentLargeFixedName
                            });

                            reject(intentBot);
                        }

                    }else{

                        reject({
                            intent: "naoentendi",
                            dialog: companyId+".naoentendi"
                        });

                        Raven.captureMessage('Empresa não deu intent', {user : {userId: userId, text: text, companyIntents : companyIntents, language: language, intentResult: response.result.metadata.intentName, companyId: companyId }}, function (err, eventId) {
                            // The message has now been sent to Sentry
                        });
                    }


                });

                request.on('error', function (error) {

                    // Raven.captureException(error);

                    Raven.captureMessage('Erro na request', {user : {userId: userId, text: text, companyIntents : companyIntents, language: language, error: error, msg: msg}}, function (err, eventId) {
                        // The message has now been sent to Sentry
                    });

                    console.log(error);
                    let intentLargeFixedName = "naoentendi";
                    let intentBot = intents.find(item => {
                        return item.intent == intentLargeFixedName
                    });

                    reject(intentBot);
                });

                request.end();
            } else {

                let intentLargeFixedName = "naoentendi_grande";
                let intentBot = intents.find(item => {
                    return item.intent == intentLargeFixedName
                });

                if (intentBot) {
                    resolve( intentBot)
                } else {
                    reject()
                }
            }

        })

    };

    resolver.tryToResolveTextByKeywords = async function(text, intents, expressions, globalCache){

        return new Promise( async (resolve, reject) => {

            let keyWordMatch = await KeywordMatcher.getIntent(text, expressions, globalCache);

            if (keyWordMatch && keyWordMatch.length > 0){

                let intentBot = intents.find(item => {
                    return item.intent == keyWordMatch[0].intent;
                });

                if (intentBot) {
                    resolve( intentBot)
                } else {
                    reject();
                }
            }
        })

    };
    return resolver;
}
