const accounting = require('accounting');

let str = ["oi", "oi", "ola", "bom dia", "boa tarde", "boa noite", "como vai", "ei", "oii", "oiii", "oiiii", "oiiiii", "oiiiiii", "oiiiiiii", "ooi", "oooi", "oooii", "ooooii", "ooooiii", "oola", "olaa", "olaaa", "hola", "holaa", "heloo", "hello", "helooo", "hellooo", "booa", "boaa",
    "tudo bem", "tudo bem", "como vai", "tudo certo", "obrigado", "certo", "entendi", "entendi mas", "certo mas", "olá!", "ola!"];


let Util = {};


Util.isPhrase = function (phrase) {
    return phrase.split(" ").length >= 3
};

function wordInString(s, words, replacement) {

    s = removeDotsAndComma(s);

    let re = new RegExp('(\\s|^)(?:' + words.join('|') + ')(?=\\s|$)');
    return s.replace(re, replacement);
}

function removeDotsAndComma(phrase) {
    let regex = /[.,]/g;
    return phrase.replace(regex, '');
}

Util.replaceIfIsPhrase = function (phrase) {
    phrase = phrase.trim();
    return Util.isPhrase(phrase) ? wordInString(phrase, str, "").trim() : phrase;
};


Util.capitalizeFirstLetter = function (string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
};


Util.dateDiffInDays = function (a, b) {
    let _MS_PER_DAY = 1000 * 60 * 60 * 24;
    // Discard the time and time-zone information.
    let utc1 = Date.UTC(a.getFullYear(), a.getMonth(), a.getDate());
    let utc2 = Date.UTC(b.getFullYear(), b.getMonth(), b.getDate());

    return Math.floor((utc2 - utc1) / _MS_PER_DAY);
};


Util.toReal = function (numero) {
    let num = numero;

    if (typeof numero === 'string') {
        num = Number.parseFloat(numero);
    }

    return accounting.formatMoney(num, 'R$ ', 2, '.', ',');
};


Util.letterCount = function (str) {
    var i = 0,
        seq = 0,
        results = [];

    while (i < str.length) {
        var current = str[i],
            next = str[i + 1];

        if (typeof results[seq] === 'undefined') {
            results[seq] = [current, 0];
        }

        results[seq][1]++;

        if (current !== next) {
            seq++;
        }

        i++;
    }

    return results;
}

Util.isNumeric = function isNumeric(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
}


Util.existsletterCount = function (str, letterMaxCount = 3) {

    let value = false;

    if (str != null) {
        let letterCount = Util.letterCount(str);
        if (letterCount && letterCount.length > 0) {

            let find = letterCount.find(item => {
                return item[1] >= letterMaxCount
            });

            value = find != null;
        }
    }
    return value;
}

// console.log(Util.existsletterCount("11", 3))

//console.log(Util.replaceIfIsPhrase('Boa noite .'))

module.exports = Util;
